#!/bin/env python
"""
 ATENVNC Connection script for Linux, Mac, Windows and others
==============================================================
 Author: Paul Banks (http://paulbanks.org)

Usage:

    python ./atenvnc.py -h hostname -p password [-P vncport]

    -h hostname      Hostname or IP address of the KE8220
    -p password      Password the KE8220 should use to connect to this host.    
    -P vncport       VNC Port that is open on THIS host. (default 5900)

"""

import getopt
import sys, struct, socket

# Import PyCypto AES library
try:
    import Crypto.Cipher.AES as aes
except ImportError:
    print """
ERROR: Could not import the AES module from PyCrypto.

You need to have the PyCrypto libraries installed to use this software!

       On Linux, this is often available as a package for your distribution.

       If you can't locate the package, then you must build from source using
       http://www.dlitz.net/software/pycrypto/

       If you're still stuck, leave a message at http://paulbanks.org/contact/
"""
    sys.exit(2)    

ATEN_KEY = "\x4A\x41\xA5\x43\xB4\xAF\x00\x00\x4C\xA7\xB6\x00\xE8\xCD\xF1\x04\xF5\x49\xBA\x06\x18\x0A\x93\x2A\x28\x20\xD0\x07\x2D\xA8\xDE\x04";

def AtenEncrypt(plaintext):

    # Create AES encryptor
    a = aes.new(ATEN_KEY, aes.MODE_CBC)
  
    # Pad packet to be multiple of 16
    padlen = 16 - len(plaintext) % 16
    if padlen==0:
        padlen = 16;
    plaintext += chr(padlen) * padlen

    # Return encrypted packet
    return a.encrypt(plaintext)

def AtenDecrypt(ciphertext):

    # Create AES encryptor
    a = aes.new(ATEN_KEY, aes.MODE_CBC)

    # Return decrypted packet
    return a.decrypt(ciphertext)

def GenerateConnectRequestPacket(host, port, password):

    # Generate plain text
    packetdata = "[Connect]" + \
        struct.pack("<i", len(host)) + host  + \
        struct.pack("<i", len(str(port))) + str(port) + \
        struct.pack("<i", len(password)) + password

    # Encrypt
    encdata = AtenEncrypt(packetdata)

    return struct.pack("<i", len(encdata)) + encdata;


def Usage():
    print __doc__

def Main():
   
    # Parse command line options
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h:p:P:")
    except getopt.GetoptError as err:
        print str(err)
        Usage()
        sys.exit(2)

    kehost = None
    vncpasswd = None
    vncport = 5900
    for o, a in opts:
        if o == "-h":
            kehost = a
        elif o == "-p":
            vncpasswd = a
        elif o == "-P":
            vncport = int(a)
    
    # Check args
    if kehost==None or vncpasswd==None:
        print "You must specify the KE Hostname and VNC password!"
        Usage()
        sys.exit(2)

    # Connect to projector adaptor
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((kehost, 1352))
    except Exception as ex:
        print "Unable to connect to KE8220 at %s: %s" % (kehost, str(ex))
        sys.exit(1)

    # Generate connection request
    packet = GenerateConnectRequestPacket(
            s.getsockname()[0], 
            vncport, 
            vncpasswd
    )

    # Send packet
    s.send(packet)
    print "Sent connection request."

    # Get reply
    replypacket = s.recv(1024)
    reply = AtenDecrypt(replypacket[4:])

if __name__=="__main__":
    Main()
   
